require 'json'
require 'digest'
require 'net/http'

module Sms
  class Taobao

    attr_accessor :app_secret, :app_key, :sms_free_sign_name, :sms_template_code, :sms_param

    def initialize(options = {})
      self.app_secret = options[:app_secret]
      self.app_key = options[:app_key]
      self.sms_free_sign_name = options[:sms_free_sign_name]
      self.sms_template_code = options[:sms_template_code]
      self.sms_param = options[:sms_param]
    end

    def singleSend(number, content = nil)
      send(number.split, content)
    end

    def batchSend(numbers, content = nil)
      send(numbers, content)
    end

    def defaultParams(numbers, content = nil)
      {
          method: 'alibaba.aliqin.fc.sms.num.send',
          app_key: self.app_key,
          timestamp: Time.now.strftime("%Y-%m-%d %H:%M:%S"),
          format: 'json',
          v: 2.0,
          sign_method: 'md5',
          sms_type: 'normal',
          sms_free_sign_name: self.sms_free_sign_name,
          rec_num: numbers.join(','),
          sms_param: (self.sms_param || {content: content}).to_json,
          sms_template_code: self.sms_template_code
      }
    end

    def generate_sign(params)
      str = params.to_a.map{|m| m.map(&:to_s).join }.sort.join
      Digest::MD5.hexdigest("#{app_secret.to_s}#{str}#{app_secret.to_s}").upcase
    end

    def send(numbers, content)
      params = defaultParams(numbers, content)
      params = params.merge(sign: generate_sign(params))
      data = request_api(params)
      result = JSON.parse(data.body)
    end

    def request_api(params)
      Net::HTTP.post_form(URI.parse('http://gw.api.taobao.com/router/rest'), params)
    end

  end
end

# options = {app_secret: '5aff003ac3c7243da65adef1e7c9c363', app_key: '23272925', sms_free_sign_name: '爱客验证码', sms_template_code: 'SMS_2575137', sms_param: {code: 'xxx', product: 'ddd'}}
# puts Sms::Taobao.new(options).singleSend('13262902619')